@@ -26,21 +26,10 @@
 #include <glib-object.h>
 #include <sys/types.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+G_BEGIN_DECLS
 
 #define GSM_TYPE_CLIENT            (gsm_client_get_type ())
-#define GSM_CLIENT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GSM_TYPE_CLIENT, GsmClient))
-#define GSM_CLIENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GSM_TYPE_CLIENT, GsmClientClass))
-#define GSM_IS_CLIENT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GSM_TYPE_CLIENT))
-#define GSM_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GSM_TYPE_CLIENT))
-#define GSM_CLIENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GSM_TYPE_CLIENT, GsmClientClass))
-
-typedef struct _GsmClient        GsmClient;
-typedef struct _GsmClientClass   GsmClientClass;
-
-typedef struct GsmClientPrivate GsmClientPrivate;
+G_DECLARE_DERIVABLE_TYPE (GsmClient, gsm_client, GSM, CLIENT, GObject)
 
 typedef enum {
         GSM_CLIENT_UNREGISTERED = 0,
@@ -62,12 +51,6 @@ typedef enum {
         GSM_CLIENT_END_SESSION_FLAG_LAST     = 1 << 2
 } GsmClientEndSessionFlag;
 
-struct _GsmClient
-{
-        GObject           parent;
-        GsmClientPrivate *priv;
-};
-
 struct _GsmClientClass
 {
         GObjectClass parent_class;
@@ -110,7 +93,6 @@ typedef enum
 
 GType                 gsm_client_error_get_type             (void);
 GQuark                gsm_client_error_quark                (void);
-
 GType                 gsm_client_get_type                   (void) G_GNUC_CONST;
 
 const char           *gsm_client_peek_id                    (GsmClient  *client);
@@ -168,8 +150,6 @@ void                  gsm_client_end_session_response       (GsmClient  *client,
                                                              gboolean    cancel,
                                                              const char *reason);
 
-#ifdef __cplusplus
-}
-#endif
+G_END_DECLS
 
 #endif /* __GSM_CLIENT_H__ */
